@@ -124,7 +124,11 @@ void pool_session_context_destroy(void)
 		pool_clear_sent_message_list();
 		free(session_context->message_list.sent_messages);
 		pool_memory_delete(session_context->memory_context, 0);
+
+		if (session_context->query_context)
+			pool_query_context_destroy(session_context->query_context);
 	}
+
 	/* XXX For now, just zap memory */
 	memset(&session_context_d, 0, sizeof(session_context_d));
 	session_context = NULL;
@@ -725,6 +729,13 @@ POOL_TRANSACTION_ISOLATION pool_get_transaction_isolation(void)
 	/* No cached data is available. Ask backend. */
 	status = do_query(MASTER(session_context->backend),
 					  "SELECT current_setting('transaction_isolation')", &res, MAJOR(session_context->backend));
+	if(status != POOL_CONTINUE)
+	{
+		pool_error("pool_get_transaction_isolation: do_query returns no rows");
+		if(res)
+			free_select_result(res);
+		return POOL_UNKNOWN;
+	}
 
 	if (res->numrows <= 0)
 	{
@@ -745,8 +756,12 @@ POOL_TRANSACTION_ISOLATION pool_get_transaction_isolation(void)
 		return POOL_UNKNOWN;
 	}
 
-	if (!strcmp(res->data[0], "read committed"))
+	if (!strcmp(res->data[0], "read uncommitted"))
+		ret = POOL_READ_UNCOMMITTED;
+	else if (!strcmp(res->data[0], "read committed"))
 		ret = POOL_READ_COMMITTED;
+	else if (!strcmp(res->data[0], "repeatable read"))
+		ret = POOL_REPEATABLE_READ;
 	else if (!strcmp(res->data[0], "serializable"))
 		ret = POOL_SERIALIZABLE;
 	else
@@ -756,7 +771,8 @@ POOL_TRANSACTION_ISOLATION pool_get_transaction_isolation(void)
 		ret = POOL_UNKNOWN;
 	}   
 
-	free_select_result(res);
+	if(res)
+		free_select_result(res);
 
 	if (ret != POOL_UNKNOWN)
 		session_context->transaction_isolation = ret;
